@@ -1,5 +1,5 @@
 /* Focus Blur -- blur with focus plug-in.
- * Copyright (C) 2002-2007 Kyoichiro Suda
+ * Copyright (C) 2002-2008 Kyoichiro Suda
  *
  * The GIMP -- an image manipulation program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
@@ -22,25 +22,14 @@
 #ifndef __FOCUSBLUR_INTERFACE_H__
 #define __FOCUSBLUR_INTERFACE_H__
 
-#include <libgimp/gimp.h>
-#include <libgimp/gimpui.h>
+#include <glib/gtypes.h>
 
 #include "focusblurtypes.h"
 
 G_BEGIN_DECLS
 
 
-gboolean        focusblur_execute               (FblurParam     *param,
-                                                 GimpPreview    *preview);
-#ifdef ENABLE_MP
-void            focusblur_execute_thread        (gpointer        data,
-                                                 gpointer        user_data);
-#endif
 gboolean        focusblur_dialog                (FblurParam     *param);
-void            focusblur_widget_set_enable_depth_map   (FblurParam     *param,
-                                                         gboolean        bool);
-void            focusblur_widget_set_enable_shine       (FblurParam     *param,
-                                                         gboolean        bool);
 
 
 G_END_DECLS
